# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import os

import maya.cmds as cmds
from PySide2 import QtWidgets

from maya.app.general import mayaMixin

from .ui import ui_form


# devkitに含まれているpyJsonAttrPatternFactoryプラグインのパス
_JSON_PLUGIN = r'C:\Program Files\Autodesk\Maya2019\devkit\plug-ins\scripted\pyJsonAttrPatternFactory.py'

# アトリビュートパターンが定義されているJsonファイル
# 定義ファイルのサンプル(C:\Program Files\Autodesk\Maya2019\devkit\plug-ins\scripted\sampleAttrPatterns.json)　+ ヘルプページに記載されている定義
_ATTR_JSON = os.path.join(os.path.dirname(__file__), 'sampleAttrPatterns.json')


class AttrPatternManager(mayaMixin.MayaQWidgetBaseMixin, QtWidgets.QWidget):
    def __init__(self, parent=None):
        super(AttrPatternManager, self).__init__(parent)

        # Plugin有効化
        if not cmds.pluginInfo(_JSON_PLUGIN, q=True, l=True):
            cmds.loadPlugin(_JSON_PLUGIN)

        # 重複したアトリビュートパターンを削除するためにdeleteAttrPatternを実行すると以降のNewScene・OpenSceneでMayaが落ちるので
        # 定義ファイルの読み込みはMaya起動中に1度だけとする
        if not cmds.listAttrPatterns():
            cmds.createAttrPatterns(pt='json', pf=_ATTR_JSON)

        self.__ui = ui_form.Ui_Form()
        self.__ui.setupUi(self)

        self.__ui.apply_btn.clicked.connect(self.__apply_ex_attrs_pattern)
        self.__ui.clear_btn.clicked.connect(self.__clear_ex_attrs)

        self.__update_ui()

    def __update_ui(self):
        """アトリビュートパターンをリスト表示
        """
        self.__ui.pattern_list.clear()
        self.__ui.pattern_list.addItems(cmds.listAttrPatterns())

    def __apply_ex_attrs_pattern(self):
        """リストで選択したアトリビュートパターンを選択しているノードに適用
        """
        # ノードのExtraAttributeをすべて削除
        self.__clear_ex_attrs()

        # アトリビュートパターンを適用してExtraAttributeを追加
        selected_nodes = cmds.ls(sl=True)
        pattern_name = self.__ui.pattern_list.currentItem().text()
        cmds.applyAttrPattern(selected_nodes, pn=pattern_name)

    def __clear_ex_attrs(self):
        """選択しているノードのExtraAttributeを全て削除
        """
        selected_nodes = cmds.ls(sl=True)
        for n in selected_nodes:
            ex_attrs = cmds.listAttr(n, ud=True) or []
            if not any(ex_attrs):
                continue

            for attr in ex_attrs:
                if cmds.attributeQuery(attr, n=n, ex=True):
                    cmds.deleteAttr('{0}.{1}'.format(n, attr))
